#include "StopAndWaitCullingManager.h"
#include "CullingLogManager.h"

namespace GtpVisibility {

//-----------------------------------------------------------------------
void StopAndWaitCullingManager::RenderScene()
{
	if (0) CullingLogManager::GetSingleton()->LogMessage("swc");

	while (!mHierarchyInterface->GetQueue()->empty())
	{
		HierarchyNode *node = mHierarchyInterface->GetQueue()->top();
		mHierarchyInterface->GetQueue()->pop();
			
		// interesting for visualization purpose
		if (mVisualizeCulledNodes)
		{
			mHierarchyInterface->SetNodeVisible(node, false);
			mHierarchyInterface->SetLastVisited(node, mHierarchyInterface->GetFrameId());
		}

		bool intersects = false;
	
		if (!mHierarchyInterface->CheckFrustumVisible(node, intersects))
		{
			++ mNumFrustumCulledNodes;
		
			if (mVisualizeCulledNodes)
			{
				mHierarchyInterface->VisualizeCulledNode(node, FRUSTUM_CULLED);
			}
		}
		// if intersects near plane => skip occlusion query because wrong results possible
		else if (intersects)
		{
			if (mVisualizeCulledNodes)
			{
				mHierarchyInterface->SetNodeVisible(node, true);
			}

			mHierarchyInterface->TraverseNode(node);
		}
		else
		{	
			++ mNumQueriesIssued;
			unsigned int visiblePixels = 0;
			const bool waitForResult = true;
			mHierarchyInterface->IssueNodeOcclusionQuery(node)->GetQueryResult(visiblePixels, waitForResult);
		
			// node visible
			if (visiblePixels > mVisibilityThreshold)
			{
				mHierarchyInterface->TraverseNode(node);
			}
			else
			{	
				++ mNumQueryCulledNodes;

				if (mVisualizeCulledNodes)
				{
					mHierarchyInterface->VisualizeCulledNode(node, QUERY_CULLED);
				}
			}
		}
	}
}
} // namespace GtpVisibility
